/*
 * This file is part of TelepathyLoggerQt
 *
 * Copyright (C) 2011 Collabora Ltd. <http://www.collabora.co.uk/>
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _TelepathyLoggerQt_init_h_HEADER_GUARD_
#define _TelepathyLoggerQt_init_h_HEADER_GUARD_

#include <TelepathyLoggerQt_export.h>

namespace Tpl
{

/*!
 * \headerfile init.h <TelepathyLoggerQt/Init>
 * \brief Initialization of the telepathy logger qt4 bindings
 */

TELEPATHY_LOGGER_QT_EXPORT void init();

} //namespace

#endif
